> {-# LANGUAGE NoMonomorphismRestriction, StandaloneDeriving #-}
>
> import Prelude hiding (cycle)
> import Data.Complex
>
> import Diagrams.Prelude hiding (conjugate)
> import Diagrams.Backend.SVG (Options(SVGOptions), SVG(SVG))
> import Text.Blaze.Svg.Renderer.Utf8 (renderSvg)
> import Data.ByteString.Lazy (writeFile)
> type DiagramSVG = Diagram SVG R2

----------------------------------------------------------------------

Primitive shapes

> data Shape 
>   =  Rectangle Double Double
>   |  Ellipse Double Double
>   |  Triangle Double

Style settings

> type Col = Colour Double
> type StyleSheet = [Styling]
> data Styling
>   =  FillColour Col
>   |  StrokeColour Col
>   |  StrokeWidth Double

Pictures (arrangements of shapes)

> data Picture 
>   =  Place StyleSheet Shape
>   |  Above Picture Picture
>   |  Beside Picture Picture

> figure :: Picture
> figure =  Place [StrokeWidth 0.1, FillColour bisque] (Ellipse 3 3) `Above`
>           Place [FillColour red, StrokeWidth 0] (Rectangle 10 1) `Above`
>           Place [FillColour red, StrokeWidth 0] (Triangle 10) `Above`
>           ( Place [FillColour blue, StrokeWidth 0] (Rectangle 1 5) `Beside`
>             Place [StrokeWidth 0] (Rectangle 2 5) `Beside`
>             Place [FillColour blue, StrokeWidth 0] (Rectangle 1 5) ) `Above`
>           ( Place [FillColour blue, StrokeWidth 0] (Rectangle 2 1) `Beside`
>             Place [StrokeWidth 0] (Rectangle 2 1) `Beside`
>             Place [FillColour blue, StrokeWidth 0] (Rectangle 2 1) ) 

----------------------------------------------------------------------

Simple transformations

> type Pos = Complex Double
> data Transform
>   =  Identity
>   |  Translate Pos
>   |  Compose Transform Transform

> transformPos :: Transform -> Pos -> Pos
> transformPos Identity       = id
> transformPos (Translate p)  = (p+)
> transformPos (Compose t u)  = transformPos t . transformPos u

----------------------------------------------------------------------

Simplified form for pictures:

> type Drawing = [ (Transform, StyleSheet, Shape) ] -- non-empty, will be centred

In order to place drawings next to each other, we'll need to compute extents.

> type Extent = (Pos,Pos) -- (lower left, upper right)
> unionExtent :: Extent -> Extent -> Extent
> unionExtent (llx1 :+ lly1, urx1 :+ ury1) (llx2 :+ lly2, urx2 :+ ury2) 
>   = (min llx1 llx2 :+ min lly1 lly2, max urx1 urx2 :+ max ury1 ury2) 

> drawingExtent :: Drawing -> Extent
> drawingExtent = foldr1 unionExtent . map getExtent where
>   getExtent (t,_,s) =  let (ll,ur) = shapeExtent s
>                        in (transformPos t ll, transformPos t ur)

> shapeExtent :: Shape -> Extent
> shapeExtent (Ellipse xr yr)  = (-(xr :+ yr), xr :+ yr)
> shapeExtent (Rectangle w h)  = ( -(w/2 :+ h/2), w/2 :+ h/2)
> shapeExtent (Triangle s)     = ( -(s/2 :+ sqrt 3 * s/4), s/2 :+ sqrt 3 * s/4)

----------------------------------------------------------------------

Now to simplify pictures into this form

> drawPicture :: Picture -> Drawing
> drawPicture (Place u s)      = drawShape u s
> drawPicture (Above p q)      = drawPicture p `aboveD` drawPicture q
> drawPicture (Beside p q)     = drawPicture p `besideD` drawPicture q

> drawShape :: StyleSheet -> Shape -> Drawing
> drawShape u s = [(Identity,u,s)]

> aboveD :: Drawing -> Drawing -> Drawing
> pd `aboveD` qd =   transformDrawing (Translate (0 :+ qury)) pd ++ 
>                    transformDrawing (Translate (0 :+ plly)) qd where
>   (pllx :+ plly, pur)  = drawingExtent pd
>   (qll, qurx :+ qury)  = drawingExtent qd

> besideD :: Drawing -> Drawing -> Drawing
> pd `besideD` qd =  transformDrawing (Translate (qllx :+ 0)) pd ++ 
>                    transformDrawing (Translate (purx :+ 0)) qd where
>   (pll, purx :+ pury)  = drawingExtent pd
>   (qllx :+ qlly, qur)  = drawingExtent qd

> transformDrawing :: Transform -> Drawing -> Drawing
> transformDrawing t = map (\ (t',u,s) -> (Compose t t',u,s))

----------------------------------------------------------------------

Finally, we should assemble our Drawing into DiagramSVG

> assemble :: Drawing -> DiagramSVG
> assemble = foldr1 atop . map (\ (t,u,s) -> transformDiagram t (diagramShape u s))

> transformDiagram :: Transform -> DiagramSVG -> DiagramSVG
> transformDiagram Identity              = id
> transformDiagram (Translate (x :+ y))  = translate (r2 (x,y))
> transformDiagram (Compose t u)         = transformDiagram t . transformDiagram u

> diagramShape :: StyleSheet -> Shape -> DiagramSVG
> diagramShape u s = shape (applyStyleSheet u) s where
>   shape f (Ellipse xr yr)  =  f (ellipseXY xr yr)
>   shape f (Rectangle w h)  =  f (rect w h)
>   shape f (Triangle s)     =  f (translate (r2 (0,-y)) (eqTriangle s))
>                               where y = s * sqrt 3 / 12

> applyStyleSheet :: StyleSheet -> (DiagramSVG -> DiagramSVG)
> applyStyleSheet = foldr (.) id . map applyStyling
> applyStyling :: Styling -> DiagramSVG -> DiagramSVG
> applyStyling (FillColour c)    = fc c
> applyStyling (StrokeColour c)  = lc c
> applyStyling (StrokeWidth w)   = lw w

...so we can render it to an SVG file

> writeSVG :: FilePath -> DiagramSVG -> IO ()
> writeSVG f = Data.ByteString.Lazy.writeFile f . renderSvg . renderDia SVG (SVGOptions (Width (fromIntegral 400))) . pad 1.1 . centerXY

----------------------------------------------------------------------

Here are the tile markings for Escher's "Square Limit".

> markingsP = [
>   [ (4:+4), (6:+0) ],
>   [ (0:+3), (3:+4), (0:+8), (0:+3) ],
>   [ (4:+5), (7:+6), (4:+10), (4:+5) ], 
>   [ (11:+0), (10:+4), (8:+8), (4:+13), (0:+16) ], 
>   [ (11:+0), (14:+2), (16:+2) ], 
>   [ (10:+4), (13:+5), (16:+4) ], 
>   [ (9:+6), (12:+7), (16:+6) ], 
>   [ (8:+8), (12:+9), (16:+8) ], 
>   [ (8:+12), (16:+10) ], 
>   [ (0:+16), (6:+15), (8:+16), (12:+12), (16:+12) ], 
>   [ (10:+16), (12:+14), (16:+13) ], 
>   [ (12:+16), (13:+15), (16:+14) ], 
>   [ (14:+16), (16:+15) ]
>   ]

> markingsQ = [
>   [ (2:+0), (4:+5), (4:+7) ],
>   [ (4:+0), (6:+5), (6:+7) ],
>   [ (6:+0), (8:+5), (8:+8) ],
>   [ (8:+0), (10:+6), (10:+9) ],
>   [ (10:+0), (14:+11) ],
>   [ (12:+0), (13:+4), (16:+8), (15:+10), (16:+16), (12:+10), (6:+7), (4:+7), (0:+8) ],
>   [ (13:+0), (16:+6) ],
>   [ (14:+0), (16:+4) ],
>   [ (15:+0), (16:+2) ],
>   [ (0:+10), (7:+11) ],
>   [ (9:+12), (10:+10), (12:+12), (9:+12) ],
>   [ (8:+15), (9:+13), (11:+15), (8:+15) ],
>   [ (0:+12), (3:+13), (7:+15), (8:+16) ],
>   [ (2:+16), (3:+13) ],
>   [ (4:+16), (5:+14) ],
>   [ (6:+16), (7:+15) ]
>   ]

> markingsR = [
>   [ (0:+12), (1:+14) ],
>   [ (0:+8), (2:+12) ],
>   [ (0:+4), (5:+10) ],
>   [ (0:+0), (8:+8) ],
>   [ (1:+1), (4:+0) ],
>   [ (2:+2), (8:+0) ],
>   [ (3:+3), (8:+2), (12:+0) ],
>   [ (5:+5), (12:+3), (16:+0) ],
>   [ (0:+16), (2:+12), (8:+8), (14:+6), (16:+4) ],
>   [ (6:+16), (11:+10), (16:+6) ],
>   [ (11:+16), (12:+12), (16:+8) ],
>   [ (12:+12), (16:+16) ],
>   [ (13:+13), (16:+10) ],
>   [ (14:+14), (16:+12) ],
>   [ (15:+15), (16:+14) ]
>   ]

> markingsS = [
>   [ (0:+0), (4:+2), (8:+2), (16:+0) ],
>   [ (0:+4), (2:+1) ],
>   [ (0:+6), (7:+4) ],
>   [ (0:+8), (8:+6) ],
>   [ (0:+10), (7:+8) ],
>   [ (0:+12), (7:+10) ],
>   [ (0:+14), (7:+13) ],
>   [ (8:+16), (7:+13), (7:+8), (8:+6), (10:+4), (16:+0) ],
>   [ (10:+16), (11:+10) ],
>   [ (10:+6), (12:+4), (12:+7), (10:+6) ],
>   [ (13:+7), (15:+5), (15:+8), (13:+7) ],
>   [ (12:+16), (13:+13), (15:+9), (16:+8) ],
>   [ (13:+13), (16:+14) ],
>   [ (14:+11), (16:+12) ],
>   [ (15:+9), (16:+10) ]
>   ]

This is the definition you'll need to draw markings.

> tile :: [[Pos]] -> DiagramSVG
> tile m = centerXY (position (map (chunk . points) m)) where
>   chunk vs  = (head vs, pathLike (0&0) False (segmentsFromVertices vs))
>   points    = map (\ (x :+ y) -> p2 (x,y))

The remainder is commented out, because it depends on features that
get introduced during the exercises.

 > fishP, fishQ, fishR, fishS :: Picture
 > fishP = Place [StrokeWidth 0.05] (Tile 16 markingsP)
 > fishQ = Place [StrokeWidth 0.05] (Tile 16 markingsQ)
 > fishR = Place [StrokeWidth 0.05] (Tile 16 markingsR)
 > fishS = Place [StrokeWidth 0.05] (Tile 16 markingsS)

 > quartet p q r s = Expand 0.5 ((p `Beside` q) `Above` (r `Beside` s))
 > cycle p = quartet p (Rot(Rot(Rot p))) (Rot p) (Rot(Rot p))
 > fishT = quartet fishP fishQ fishR fishS
 > fishU = cycle (Rot fishQ)
 > blank = Place [StrokeWidth 0] (Rectangle 16 16)
 > side1 = quartet blank blank (Rot fishT) fishT
 > side2 = quartet side1 side1 (Rot fishT) fishT
 > corner1 = quartet blank blank blank fishU
 > corner2 = quartet corner1 side1 (Rot side1) fishU
 > pseudocorner = quartet corner2 side2 (Rot side2) (Rot fishT)
 > pseudolimit = cycle pseudocorner
 > nonet p1 p2 p3 p4 p5 p6 p7 p8 p9 = Expand (1/3) (
 >   (p1 `Beside` p2 `Beside` p3) `Above` 
 >   (p4 `Beside` p5 `Beside` p6) `Above`
 >   (p7 `Beside` p8 `Beside` p9))
 > corner = nonet corner2 side2 side2 (Rot side2) fishU (Rot fishT) (Rot side2) (Rot fishT) (Rot fishQ)
 > squarelimit = cycle corner

----------------------------------------------------------------------

For testing purposes

> deriving instance Show Shape
> deriving instance Show Styling
> deriving instance Show Picture
> deriving instance Show Transform
